#!/usr/bin/env python
# Copyright (c) 2004, 2009, Fredrik Arnerup, see COPYING
# SlideRule A slide rule simulator

version = "0.2"

# boring theme:
## bg_color = "light gray"
## shadow_color = "dark gray"
## marker_color = "gray"

bg_color = "#fff0aa"
shadow_color = "#ddc099"
marker_color = "#ffffdd"

import pygtk
pygtk.require('2.0')
import gtk, gnomecanvas
from math import log10

class Part(gtk.Window):
    def __init__(self, width, height, aa=False):
        gtk.Window.__init__(self, gtk.WINDOW_POPUP)
        self.width, self.height = width, height
        self.resize(width, height)
        self.moving = False;
        self.canvas = gnomecanvas.Canvas(aa = aa)
        self.add(self.canvas)
        self.canvas.set_size_request(width, height)
        self.canvas.set_scroll_region(0,0, width, height)
        self.canvas.connect("button_press_event", self.press)
        self.canvas.connect("button_release_event", self.release)
        self.canvas.connect("motion_notify_event", self.motion)

    def add_default_bg(self):
        self.canvas.root().add(gnomecanvas.CanvasRect,
                               x1 = 0, y1 = 0,
                               x2 = self.width - 1, y2 = self.height - 1,
                               outline_color = "black",
                               fill_color = bg_color)

    def add_ticks(self, x1, x2, y, height, down, finish=False):
        def tick(x, hf):
            self.canvas.root().add(gnomecanvas.CanvasRect,
                                   x1 = x, x2 = x,
                                   y1 = y, y2 = y + hf * f * height,
                                   outline_color = "black")
        def logscale(x):
            return x1 + c * log10(x)
        c = x2 - x1
        f = down and 1 or -1
        end = finish and 11 or 10
        for x in range(1, end):
            number = (x == 10) and "1" or chr(ord('0') + x)
            xx = logscale(x)
            xn = logscale(x + 1)
            dx = xn - xx
            tick(xx, 1)
            self.canvas.root().add(gnomecanvas.CanvasText,
                                   x = xx, y = y + f * 1.5 * height,
                                   fill_color = "black", family = "sans",
                                   size = 1000 * height / 2,
                                   text = number)
            if x == 10:
                break;
            if dx > 40: steps = 10
            elif dx > 15: steps = 5
            else: steps = 1

            step = 1.0 / steps
            for ss in range(1, steps):
                xx = logscale(x + ss * step)
                if steps == 10 and ss == 5:
                    tick(xx, 1)
                else:
                    tick(xx, 0.5)
            
    def press(self, widget, event):
        if event.button == 1:
            self.moving = True
            self.sx, self.sy = event.x_root, event.y_root

    def release(self, widget, event):
        self.moving = False
        
    def motion(self, widget, event):
        pass

class Slider(Part):
    def __init__(self, width, height, ruler):
        Part.__init__(self, width, height)
        self.ruler = ruler
        self.pos = 0
        self.add_default_bg()


    def motion(self, widget, event):
        if self.moving:
            x, y = self.window.get_origin()
            self.move(int(x + event.x_root - self.sx), y)
            x, y = self.window.get_origin()
            self.pos = x - ruler.window.get_origin()[0]
            self.sx, self.sy = event.x_root, event.y_root
            
    def get_pos(self): return self.pos
            
class Marker(Slider):
    def __init__(self, width, height, ruler):
        def _set(data, x, y, clear=False):
            ibyte = y * linesize + x / 8
            ibit = x % 8
            mask = 0x01 << ibit
            if clear:
                data[ibyte] = chr(ord(data[ibyte]) & ~mask)
            else:
                data[ibyte] = chr(ord(data[ibyte]) | mask)
        Part.__init__(self, width, height, aa=False)
        bx = 10
        by = (height - ruler.height) / 2 + bx
        self.canvas.root().add(gnomecanvas.CanvasRect,
                               x1 = 0, y1 = 0, x2 = width, y2 = height,
                               fill_color = "black")
        self.canvas.root().add(gnomecanvas.CanvasRect,
                               x1 = 1, y1 = 1, x2 = width - 2, y2 = height - 2,
                               fill_color = marker_color)
        self.canvas.root().add(gnomecanvas.CanvasRect,
                               x1 = bx - 2, y1 = by - 2,
                               x2 = width - bx + 1, y2 = height - by + 1,
                               fill_color = "black")
        self.canvas.root().add(gnomecanvas.CanvasText,
                               x = width - 4, y = 2,
                               fill_color = "red", family="monospace",
                               size = 1000 * by / 3 ,
                               anchor = gtk.ANCHOR_NORTH_EAST,
                               text = "SlideRule    ")
        self.canvas.root().add(gnomecanvas.CanvasText,
                               x = width - 4, y = 2,
                               fill_color = "black", family="monospace",
                               size = 1000 * by / 3,
                               anchor = gtk.ANCHOR_NORTH_EAST,
                               text = version)
        self.pos = ruler.margin - (width + 1) / 2
        linesize = (width + 7) / 8 
        size = height * linesize
        data = ["\xff"] * size
        for x in range(bx, width - bx):
            for y in range(by, height - by):
                _set(data, x, y, True)
        x = width / 2
        for y in range(0, height):
            _set(data, x, y)
        data = "".join(data)
        bitmap = gtk.gdk.bitmap_create_from_data(self.window, data,
                                                 width, height)
        self.shape_combine_mask(bitmap, 0, 0)

class Ruler(Part):
    def __init__(self, width, height):
        Part.__init__(self, width, height)
        self.slider_height = self.height / 2
        self.marker_height = 5 * self.height / 4
        margin = self.margin = width / 15
        scale_width = self.scale_width = width - 2 * margin
        self.slider = Slider(self.width, self.slider_height, self)
        self.slider.set_transient_for(self)
        self.marker = Marker(self.width / 5, self.marker_height, self)
        self.marker.set_transient_for(self.slider)
        self.add_default_bg()
        y1 = self.slider_height / 2
        y2 = self.slider_height + self.slider_height / 2 - 1
        self.canvas.root().add(gnomecanvas.CanvasRect,
                               x1 = 0, y1 = y1,
                               x2 = width - 1,
                               y2 = y2,
                               outline_color = "black",
                               fill_color = shadow_color)
        self.slider.add_ticks(margin, margin + scale_width / 2,
                              0, self.slider_height / 5, True)
        self.slider.add_ticks(margin + scale_width  / 2, margin + scale_width,
                              0, self.slider_height / 5, True, True)
        self.slider.add_ticks(margin, margin + scale_width,
                              self.slider_height, self.slider_height / 5,
                              False, True)

        self.add_ticks(margin, margin + scale_width / 2,
                       y1, self.slider_height / 5, False)
        self.add_ticks(margin + scale_width  / 2, margin + scale_width,
                       y1, self.slider_height / 5, False, True)
        self.add_ticks(margin, margin + scale_width,
                       y2, self.slider_height / 5, True, True)
        
        self.close_button = gtk.Button()
        bw = 10; os = 5
        self.canvas.root().add(gnomecanvas.CanvasWidget,
                               x=width - os - bw, y = os,
                               width = bw, height = bw,
                               widget = self.close_button)
        self.close_button.show()
        self.close_button.connect("clicked", self.quit)
        self.move(100, 100)
        self.realize()
        self.show_all()
        self.slider.show_all()
        self.marker.show_all()
        self.adjust_slider()

    def quit(self, button):
        gtk.main_quit()

    def adjust_slider(self):
        x, y = self.window.get_origin()
        self.slider.move(x + self.slider.get_pos(), y + self.slider_height / 2)
        self.marker.move(x + self.marker.get_pos(),
                         y + (self.height - self.marker_height) / 2 )

    def motion(self, widget, event):
        if self.moving:
            x, y = self.window.get_origin()
            self.move(int(x + event.x_root - self.sx),
                      int(y + event.y_root - self.sy))
            self.adjust_slider()
            self.sx, self.sy = event.x_root, event.y_root


if __name__ == "__main__":
    width =  8 * gtk.gdk.screen_height() / 10
    height = width / 5
    ruler = Ruler(width, height)
    gtk.main()
